/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : mardi 01 aot 2006 - 17h48
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : Threads.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "Threads.h"
#include "UINotify.h"
#include "ThreadMailChecker.h"
#include "ThreadFreeGoInformations.h"
#include "main.h"

#include <QThread>
#include <QSet>
#include <QDateTime>

QSet<QThread*> mThreads;

void Threads::init()
{
	// register MailChecker Thread
	add( ThreadMailChecker::self() );
	QObject::connect( ThreadMailChecker::self(), SIGNAL( alert( const QString& ) ), Notify::self(), SLOT( alert( const QString& ) ) );
	QObject::connect( ThreadMailChecker::self(), SIGNAL( newEmails( const QString&, int ) ), Notify::self(), SLOT( newEmails( const QString&, int ) ) );

	// register FreeGoInformations Thread
	add( ThreadFreeGoInformations::self() );
	QObject::connect( ThreadFreeGoInformations::self(), SIGNAL( alert( const QString& ) ), Notify::self(), SLOT( alert( const QString& ) ) );
	QObject::connect( ThreadFreeGoInformations::self(), SIGNAL( programUpdate( int, const QDateTime&, const QString& ) ), Notify::self(), SLOT( programUpdate( int, const QDateTime&, const QString& ) ) );
	QObject::connect( ThreadFreeGoInformations::self(), SIGNAL( programUpdating( int, const QDateTime&, const QString& ) ), Notify::self(), SLOT( programUpdating( int, const QDateTime&, const QString& ) ) );
	QObject::connect( ThreadFreeGoInformations::self(), SIGNAL( freeboxUpdate( int, const QDateTime&, const QString& ) ), Notify::self(), SLOT( freeboxUpdate( int, const QDateTime&, const QString& ) ) );
	QObject::connect( ThreadFreeGoInformations::self(), SIGNAL( freeActualityUpdate( int, const QDateTime&, const QString& ) ), Notify::self(), SLOT( freeActualityUpdate( int, const QDateTime&, const QString& ) ) );
	QObject::connect( ThreadFreeGoInformations::self(), SIGNAL( teamMessageUpdate( int, const QDateTime&, const QString& ) ), Notify::self(), SLOT( teamMessageUpdate( int, const QDateTime&, const QString& ) ) );
	QObject::connect( ThreadFreeGoInformations::self(), SIGNAL( freegoAnnonce( int, const QDateTime&, const QString&, const QString& ) ), Notify::self(), SLOT( freegoAnnonce( int, const QDateTime&, const QString&, const QString& ) ) );
	QObject::connect( ThreadFreeGoInformations::self(), SIGNAL( newMessages( int ) ), Notify::self(), SLOT( newMessages( int ) ) );

	// starts threads
	start();
}

void Threads::start()
{
	foreach ( QThread* t, mThreads )
		if ( !t->isRunning() )
			t->start();
}

void Threads::stop()
{
	foreach ( QThread* t, mThreads )
	{
		if ( t->isRunning() )
		{
			t->quit();
			t->terminate();
			t->wait( 1000 );
		}
	}
}

void Threads::add( QThread* t )
{
	if ( !mThreads.contains( t ) )
		mThreads.insert( t );
}

void Threads::remove( QThread* t )
{
	if ( mThreads.contains( t ) )
		mThreads.remove( t );
}

void Threads::clean()
{
	stop();
	foreach( QThread* t, mThreads )
		delete t;
}
