/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : lundi 04 septembre 2006 - 19h17
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UIConfigure.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIConfigure.h"
#include "Options.h"
#include "misc.h"
#include "main.h"

#ifdef DIALUPCONNECTION
#include <QProcess>
#ifdef Q_OS_WIN
extern int etablir_connexion( char* username, char* password );
#endif
#endif

QPointer<UIConfigure> UIConfigure::mSelf = 0L;

UIConfigure* UIConfigure::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UIConfigure( p );
	return mSelf;
}

UIConfigure::UIConfigure( QWidget* p )
	: fgDialog( p )
{
	setupUi( this );
	fixMacWidgets();
	updateConnexionButton();

	QList<fgLabel*> buttons = findChildren<fgLabel*>( QRegExp( "fgMenu" ) );
	foreach ( fgLabel* button, buttons )
		connect( button, SIGNAL( clicked( const int ) ), this, SLOT( fgMenu_clicked( const int ) ) );

	loadFreeAccounts();
	loadSettings();
	fgMenu_clicked( 0 );
}

UIConfigure::~UIConfigure()
{
	saveFreeAccounts();
}

void UIConfigure::fgMenu_clicked( int i )
{
	QList<fgLabel*> buttons = findChildren<fgLabel*>( QRegExp( "fgMenu" ) );
	foreach ( fgLabel* button, buttons )
	{
		button->setToggled( button->data().toInt() == i );
		if ( sender() && button->data().toInt() == i )
			button->setToggled( false );
	}
	swPages->setCurrentIndex( i );
	
	// show / hide buttons
	switch ( i )
	{
		case 0: // Comptes
			fgAddUser->show();
			fgRemoveUser->show();
			fgCancel->hide();
			fgSave->hide();
			fgConnexion->hide();
			break;
		case 1: // Options
			fgAddUser->hide();
			fgRemoveUser->hide();
			fgCancel->show();
			fgSave->show();
			fgConnexion->hide();
			break;
		case 2: // Proxy
			fgAddUser->hide();
			fgRemoveUser->hide();
			fgCancel->show();
			fgSave->show();
			fgConnexion->hide();
			break;
		case 3: // Secours
			fgAddUser->hide();
			fgRemoveUser->hide();
			fgCancel->hide();
			fgSave->hide();
			fgConnexion->show();
			break;
	}
}

void UIConfigure::loadFreeAccounts()
{
	QListWidgetItem* it = 0;
	// Connexion Types
	cbFreeConnexion->clear();
	cbFreeConnexion->addItem( QString::fromUtf8( "Freebox en zone dégroupée" ), FreeBoxTotal );
	cbFreeConnexion->addItem( QString::fromUtf8( "Freebox sur numéro inactif "), FreeBoxNDI );
	cbFreeConnexion->addItem( QString::fromUtf8( "Freebox en IP/ADSL" ), FreeBoxIP );
	cbFreeConnexion->addItem( QString::fromUtf8( "Total Freebox" ), FreeBoxIPTotal );
	// Free Accounts
	const QString currentLogin = misc::identify().Login;
	QListWidgetItem* currentItem = 0;
	QList<FreeAccount> accounts = misc::freeAccountsList();
	lwFreeAccounts->clear();
	foreach ( const FreeAccount fa, accounts )
	{
		it = new QListWidgetItem( lwFreeAccounts );
		it->setText( fa.Login );
		it->setData( Qt::UserRole, QVariant::fromValue( fa ) );
		// set current if needed
		if ( fa.Login == currentLogin )
			currentItem = it;
	}
	// selecting correct free account if needed
	if ( !currentItem && lwFreeAccounts->count() )
		currentItem = lwFreeAccounts->item( 0 );
	if ( currentItem )
	{
		lwFreeAccounts->setCurrentItem( currentItem );
		lwFreeAccounts->scrollToItem( currentItem );
		on_lwFreeAccounts_itemActivated( currentItem );
	}
}

void UIConfigure::saveFreeAccounts()
{
	// save free accounts
	QListWidgetItem* it;
	Options::current()->beginWriteArray( "FreeAccounts", lwFreeAccounts->count() );
	for ( int i = 0; i < lwFreeAccounts->count(); i++ )
	{
		Options::current()->setArrayIndex( i );
		it = lwFreeAccounts->item( i );
		FreeAccount fa = it->data( Qt::UserRole ).value<FreeAccount>();
		Options::current()->setValue( "UserName", fa.Login );
		Options::current()->setValue( "Password", misc::encrypt( fa.Password ) );
		Options::current()->setValue( "Connexion", fa.Connexion );
	}
	Options::current()->endArray();
	// save default free account
	it = lwFreeAccounts->currentItem();
	FreeAccount fa;
	if ( it )
		fa = it->data( Qt::UserRole ).value<FreeAccount>();
	Options::current()->setValue( "/FreeGo/Identify/Login", fa.Login );
	Options::current()->setValue( "/FreeGo/Identify/Password", misc::encrypt( fa.Password ) );
	Options::current()->setValue( "/FreeGo/Identify/Connexion", fa.Connexion );
}

void UIConfigure::loadSettings()
{
	// Proxy
	cbUseProxy->setToggled( Options::current()->value( "/FreeGo/Proxy/Enabled", false ).toBool() );
	leProxyHost->setText( Options::current()->value( "/FreeGo/Proxy/Host" ).toString() );
	sbProxyPort->setValue( Options::current()->value( "/FreeGo/Proxy/Port" ).toInt() );
	leProxyLogin->setText( Options::current()->value( "/FreeGo/Proxy/Login" ).toString() );
	leProxyPassword->setText( Options::current()->value( "/FreeGo/Proxy/Password" ).toString() );
	// CP
	// Options
	cbMonitorAutoStart->setToggled( Options::current()->value( "/FreeGo/Options/Monitor/AutoStart", false ).toBool() );
	cbMonitorAlwaysOnTop->setToggled( Options::current()->value( "/FreeGo/Options/Monitor/AlwaysOnTop", false ).toBool() );
	sbMonitorOpacity->setValue( Options::current()->value( "/FreeGo/Options/Monitor/Opacity", 100 ).toInt() );
	cbOptionsAutoStart->setToggled( Options::current()->value( "/FreeGo/Options/AutoStart", false ).toBool() );
	cbMode->setCurrentIndex( Options::current()->value( "/FreeGo/Options/AutoStartMode", 0 ).toInt() );
	if ( Options::current()->value( "/FreeGo/Options/ShowSubscribe", false ).toBool() )
		cbShowSubscribe->setCurrentIndex( 1 );
	// Notifications
	cbNotificationFreeGo->setToggled( Options::current()->value( "/FreeGo/Notifications/FreeGo", false ).toBool() );
	cbNotificationRepondeur->setToggled( Options::current()->value( "/FreeGo/Notifications/Repondeur", true ).toBool() );
	cbNotificationFreeBox->setToggled( Options::current()->value( "/FreeGo/Notifications/FreeBox", true ).toBool() );
	cbNotificationTeam->setToggled( Options::current()->value( "/FreeGo/Notifications/Message", true ).toBool() );
	cbNotificationActuality->setToggled( Options::current()->value( "/FreeGo/Notifications/Actuality", true ).toBool() );
	cbNotificationEmails->setToggled( Options::current()->value( "/FreeGo/Notifications/Emails", true ).toBool() );
	// Secours
}

bool UIConfigure::saveSettings()
{
	if ( !lwFreeAccounts->currentItem() )
	{
		UIMAIN->alert( "Veuillez séléctionner le compte Free courant" );
		return false;
	}
	// Proxy
	Options::current()->setValue( "/FreeGo/Proxy/Enabled", cbUseProxy->toggled() );
	Options::current()->setValue( "/FreeGo/Proxy/Host", leProxyHost->text() );
	Options::current()->setValue( "/FreeGo/Proxy/Port", sbProxyPort->value() );
	Options::current()->setValue( "/FreeGo/Proxy/Login", leProxyLogin->text() );
	Options::current()->setValue( "/FreeGo/Proxy/Password", leProxyPassword->text() );
	// CP
	// Options
	Options::current()->setValue( "/FreeGo/Options/Monitor/AutoStart", cbMonitorAutoStart->toggled() );
	Options::current()->setValue( "/FreeGo/Options/Monitor/AlwaysOnTop", cbMonitorAlwaysOnTop->toggled() );
	Options::current()->setValue( "/FreeGo/Options/Monitor/Opacity", sbMonitorOpacity->value() );
	Options::current()->setValue( "/FreeGo/Options/AutoStart", cbOptionsAutoStart->toggled() );
	Options::current()->setValue( "/FreeGo/Options/AutoStartMode", cbMode->currentIndex() );
	misc::setAutoStart( cbOptionsAutoStart->toggled() );
	Options::current()->setValue( "/FreeGo/Options/ShowSubscribe", cbShowSubscribe->currentIndex() == 1 );
	// Notifications
	Options::current()->setValue( "/FreeGo/Notifications/FreeGo", cbNotificationFreeGo->toggled() );
	Options::current()->setValue( "/FreeGo/Notifications/Repondeur", cbNotificationRepondeur->toggled() );
	Options::current()->setValue( "/FreeGo/Notifications/FreeBox", cbNotificationFreeBox->toggled() );
	Options::current()->setValue( "/FreeGo/Notifications/Message", cbNotificationTeam->toggled() );
	Options::current()->setValue( "/FreeGo/Notifications/Actuality", cbNotificationActuality->toggled() );
	Options::current()->setValue( "/FreeGo/Notifications/Emails", cbNotificationEmails->toggled() );
	// Secours
	//
	return true;
}

void UIConfigure::accept()
{
	saveSettings();
}

void UIConfigure::on_lwFreeAccounts_itemActivated( QListWidgetItem* it )
{
	if ( it )
	{
		FreeAccount fa = it->data( Qt::UserRole ).value<FreeAccount>();
		leFreeLogin->setText( fa.Login );
		leFreePassword->setText( fa.Password );
		cbFreeConnexion->setCurrentIndex( cbFreeConnexion->findData( fa.Connexion ) );
	}
	else
	{
		leFreeLogin->clear();
		leFreePassword->clear();
		cbFreeConnexion->setCurrentIndex( -1 );
	}
}

void UIConfigure::on_fgAddUser_clicked()
{
	if ( !leFreeLogin->text().isEmpty() && !leFreePassword->text().isEmpty() && cbFreeConnexion->currentIndex() != -1 )
	{
		// check if item already exists
		QList<QListWidgetItem*> items;
		QListWidgetItem* it;
		items = lwFreeAccounts->findItems( leFreeLogin->text(), Qt::MatchExactly );
		if ( items.count() )
			it = items.at( 0 );
		else
			it = new QListWidgetItem( leFreeLogin->text(), lwFreeAccounts );
		FreeAccount fa = it->data( Qt::UserRole ).value<FreeAccount>();
		fa.Login = leFreeLogin->text();
		fa.Password = leFreePassword->text();
		fa.Connexion = (ConnexionFreeBox)cbFreeConnexion->itemData( cbFreeConnexion->currentIndex() ).toInt();
		it->setData( Qt::UserRole, QVariant::fromValue( fa ) );
		lwFreeAccounts->setCurrentItem( it );
		lwFreeAccounts->scrollToItem( it );
	}
}

void UIConfigure::on_fgRemoveUser_clicked()
{
	QListWidgetItem* it = lwFreeAccounts->currentItem();
	if ( !it )
		return;
	misc::freeRemoveAccount( it->text() );
	delete it;
	on_lwFreeAccounts_itemActivated( lwFreeAccounts->currentItem() );
}

void UIConfigure::updateConnexionButton()
{
	if ( !Options::current()->value( "/FreeGo/Options/Secours" ).toBool() )
	{
		fgConnexion->setButtonNormalPixmap( QPixmap( ":/configure/Pictures/configure/btn-seconnecter01.png" ) );
		fgConnexion->setButtonPushedPixmap( QPixmap( ":/configure/Pictures/configure/btn-seconnecter02.png" ) );
	}
	else
	{
		fgConnexion->setButtonNormalPixmap( QPixmap( ":/configure/Pictures/configure/btn-deconnecter01.png" ) );
		fgConnexion->setButtonPushedPixmap( QPixmap( ":/configure/Pictures/configure/btn-deconnecter02.png" ) );
	}
}

void UIConfigure::on_fgConnexion_clicked()
{
#ifdef DIALUPCONNECTION
	QString message;
#ifdef Q_OS_WIN
	QProcess rasdial;
	QByteArray out;

	QString login = misc::identify().Login;
	char* userAccount = login.toAscii().data();

	switch ( etablir_connexion( userAccount, userAccount ) )
	{
		case -1:
			message = "Aucune connection trouvée";
			break;
		case 0:
			if ( !Options::current()->value( "/FreeGo/Options/Secours" ).toBool() )
			{
				rasdial.startDetached( "rasdial", QStringList() << "FreegoSecours" << login << login );
				Options::current()->setValue( "/FreeGo/Options/Secours", true );
			}
			else
			{
				rasdial.startDetached( "rasdial", QStringList() << "/disconnect" );
				Options::current()->setValue( "/FreeGo/Options/Secours", false );
			}
			break;
		case 1:
			message = "Connexion établie";
			break;
	}
#else
	message = QString::fromUtf8( "La connexion de secours n'est pas encore gérée pour cette platforme." );
#endif
	updateConnexionButton();
	if ( !message.isEmpty() )
		UIMAIN->alert( message );
#endif
}
