/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : mardi 25 juillet 2006 - 22h51
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox gmail.com )
* FILENAME     : UIInvoice.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIInvoice.h"
#include "ChannelsGrabber.h"
#include "main.h"
#include "misc.h"
#include "Options.h"
//
#include <QShowEvent>
#include <QHeaderView>
#include <QFileDialog>
#include <QDate>
//
QPointer<UIInvoice> UIInvoice::mSelf = 0L;
const QString mInvoiceSavePathSettings = "Invoices/Path";
//
QString fixMontant( QString c )
{
	if ( c.isEmpty() )
		c = "0,000";
	bool ok;
	c.toDouble( &ok );
	if ( ok )
		c.append( QString::fromUtf8( " €" ) ).replace( '.', ',' );
	return c;
}
//
bool fakeLessThan( QTreeWidgetItem* it1, QTreeWidgetItem* it2 )
{
	int val1 = QUrl( it1->data( 0, Qt::UserRole ).toString() ).queryItemValue( "mois" ).toInt();
	int val2 = QUrl( it2->data( 0, Qt::UserRole ).toString() ).queryItemValue( "mois" ).toInt();
	return val1 > val2;
}
//
QString removeEuros( QString string )
{
	string.replace( "&euro;", QString::null );
	string.replace( "Euros", QString::null );
	string.replace( "Euro", QString::null );
	return string;
}
//
UIInvoice* UIInvoice::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UIInvoice( p );
	return mSelf;
}
//
UIInvoice::UIInvoice( QWidget* p )
	: fgDialog( p )
{
	setupUi( this );
	fixMacWidgets();
	// invoices
	twInvoices->headerItem()->setHidden( true );
	twInvoices->header()->resizeSection( 0, 150 );
	twInvoices->header()->resizeSection( 1, twInvoices->viewport()->width() -150 );
	twInvoices->setSelectionMode( QAbstractItemView::ExtendedSelection );
	twInvoices->setContextMenuPolicy( Qt::ActionsContextMenu );
	QAction* action = new QAction( "Enregistrer...", twInvoices );
	twInvoices->addAction( action );
	connect( action, SIGNAL( triggered() ), this, SLOT( saveInvoices() ) );
	// telephony
	lLine1->setVisible( false );
	lLine2->setVisible( false );
	lMobiles1->setVisible( false );
	lMobiles2->setVisible( false );
	lSpecial1->setVisible( false );
	lSpecial2->setVisible( false );
	lInternational1->setVisible( false );
	lInternational2->setVisible( false );
	lTotal1->setVisible( false );
	lTotal2->setVisible( false );
	// television
	twTelevision->headerItem()->setHidden( true );
	twTelevision->header()->resizeSection( 0, 220 );
	twTelevision->header()->resizeSection( 1, twTelevision->viewport()->width() -220 );
	// change mode
	switch ( misc::identify().Connexion )
	{
		case FreeBoxNDI:
		case FreeBoxTotal:
			changeMode( dmTelevision );
			break;
		case FreeBoxIP:
		case FreeBoxIPTotal:
			changeMode( dmTelephony );
			break;
	}
	// request login into free.fr
	REQUESTER->login( this, "readLogin" );
	// connections
	connect( REQUESTER, SIGNAL( customRequestFinished( int, bool, QIODevice* ) ), this, SLOT( customRequestFinished( int, bool, QIODevice* ) ) );
}
//
void UIInvoice::changeMode( const DialogMode& m )
{
	mMode = m;
	switch ( mMode )
	{
		case dmInvoice:
			setBackgroundPixmap( QPixmap( ":/facture/Pictures/facture/invoice01.png" ) );
			break;
		case dmTelephony:
			setBackgroundPixmap( QPixmap( ":/facture/Pictures/facture/invoice02.png" ) );
			break;
		case dmTelevision:
			setBackgroundPixmap( QPixmap( ":/facture/Pictures/facture/invoice03.png" ) );
			break;
	}
	fgClose->move( width() -( fgClose->width() +5 ), 4 );
	setWidgetTransparent( twInvoices, backgroundPixmap() );
	setWidgetTransparent( twTelevision, backgroundPixmap() );
}
//
void UIInvoice::readLogin( QSettings*, bool b )
{
	if ( b )
	{
		UIMAIN->alert( "Impossible de vous identifiez sur free, annulation" );
		deleteLater();
		return;
	}
	// invoices
	mUrlInvoices = REQUESTER->freeBaseUrl().append( "conso/liste-factures.pl" );
	REQUESTER->requestUrl( this, "readInvoices", mUrlInvoices, "GET", pHttpRequest::rtByteArray );
	// telephony
	if ( mMode == dmTelephony || mMode == dmTelevision )
	{
		mUrlTelephony = REQUESTER->freeBaseUrl().append( "conso/suivi_conso_tel.pl" );
		REQUESTER->requestUrl( this, "readTelephony", mUrlTelephony, "GET", pHttpRequest::rtByteArray );
	}
	// television
	if ( mMode == dmTelevision )
	{
		mUrlTelevision = REQUESTER->freeBaseUrl().append( "conso/suivi_conso_tv.pl" );
		REQUESTER->requestUrl( this, "readTelevision", mUrlTelevision, "GET", pHttpRequest::rtByteArray );
	}
}
//
void UIInvoice::readInvoices( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( QString( "An error occurs getting file: %1" ).arg( mUrlInvoices ) );
		return;
	}
	
	// parse
	QString ba = removeEuros( QString::fromLatin1( buffer ) );
	
	QString key = "Liste de vos factures";
	int pos = ba.indexOf( key );
	ba = ba.mid( pos +key.length() );
	
	key = "<p>";
	pos = ba.indexOf( key );
	ba = ba.mid( pos +key.length() );
	
	key = "</div>";
	pos = ba.indexOf( key );
	ba = ba.left( pos ).trimmed();
	
	QRegExp rx( "<a .*>(.*)</a>" );
	rx.setMinimal( true );
	pos = 0;
	QHash<QString, QString> mValues;
	QList<QTreeWidgetItem*> mItems;
	QTreeWidgetItem* mLastItem = 0;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// parse tag
		mValues = misc::parseHTMLTag( rx.cap( 0 ).simplified().trimmed() );
		//
		bool isInvoice = mValues[ "href" ].startsWith( "facture" );
		bool isYear;
		mValues[ "#Content" ].right( 4 ).toInt( &isYear);
		isYear = isYear && isInvoice;
		if ( !isInvoice )
		{
			// phone details
			if ( mValues[ "href" ].startsWith( "suivi_conso_tel" ) )
				fgShowDetails->setData( mValues[ "href" ] );
			continue;
		}
		// create item
		mLastItem = mItems.count() > 0 ? mItems.last() : 0;
		QTreeWidgetItem* it = isYear ? new QTreeWidgetItem : mLastItem;
		if ( it != mLastItem )
			mItems << it;
		if ( isYear )
		{
			it->setText( 0, mValues[ "#Content" ] );
			it->setData( 0, Qt::UserRole, mValues[ "href" ] );
			// last invoice tooltip
			if ( mLastItem && mLastItem->text( 1 ).isEmpty() )
			{
				mLastItem->setTextAlignment( 1, Qt::AlignRight );
				mLastItem->setText( 1, QString::fromUtf8( "Non facturé" ) );
			}
		}
		else if ( it )
		{
			it->setTextAlignment( 1, Qt::AlignRight );
			it->setText( 1, fixMontant( mValues[ "#Content" ] ) );
		}
		// remember last item
		mLastItem = it;
	}
	// update last item if needed
	if ( mLastItem && mLastItem->text( 1 ).isEmpty() )
	{
		mLastItem->setTextAlignment( 1, Qt::AlignRight );
		mLastItem->setText( 1, QString::fromUtf8( "Non facturé" ) );
	}
	// sort items, before adding them to listwidget
	qSort( mItems.begin(), mItems.end(), fakeLessThan );
	foreach ( QTreeWidgetItem* item, mItems )
		twInvoices->addTopLevelItem( item );
}
//
void UIInvoice::readTelephony( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( QString( "An error occurs getting file: %1" ).arg( mUrlTelephony ) );
		return;
	}
	
	// parse
	QString ba = removeEuros( QString::fromLatin1( buffer ) );
	
	QString key = "<div  id=\"content\" class=\"abonnement\">";
	int pos = ba.indexOf( key );
	ba = ba.mid( pos );
	
	key = "<div class=\"clearer\">";
	pos = ba.indexOf( key );
	ba = ba.left( pos ).trimmed();
	
	const QString noConso = "Pas de consommation disponible.";
	const bool isAvailable = !ba.contains( noConso );
	
	if ( !isAvailable )
	{
		lLine1->setVisible( true );
		lLine1->setText( noConso );
		return;
	}
	
	// update gui
	lLine1->setVisible( true );
	lLine2->setVisible( true );
	lMobiles1->setVisible( true );
	lMobiles2->setVisible( true );
	lSpecial1->setVisible( true );
	lSpecial2->setVisible( true );
	lInternational1->setVisible( true );
	lInternational2->setVisible( true );
	lTotal1->setVisible( true );
	lTotal2->setVisible( true );
	
	// get phone
	QRegExp rx( "\\d\\d \\d\\d \\d\\d \\d\\d \\d\\d|\\d\\d/\\d\\d/\\d\\d\\d\\d" );
	rx.setMinimal( true );
	pos = 0;
	QString line;
	QStringList dates;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// line
		if ( rx.matchedLength() == 10 )
			dates << rx.cap( 0 );
		else
			line = rx.cap( 0 );
	}
	
	lPeriode->setText( QString( "Du %1" ).arg( dates.join( " au " ) ) );
	lLine2->setText( line );
	
	rx = QRegExp( "<tr>(.*)</tr>" );
	rx.setMinimal( true );
	pos = 0;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// get tr tag
		const QString s = rx.cap( 0 ).simplified();
		// get summary
		QRegExp rxSum( ">(([\\d.]+))<" );
		QString sum;
		if ( rxSum.indexIn( s ) != -1 )
			sum = fixMontant( rxSum.cap( 1 ) );
		// check
		if ( s.contains( "Vers l'Etranger" ) )
			lInternational2->setText( sum );
		else if ( s.contains( "Vers les mobiles" ) )
			lMobiles2->setText( sum );
		else if ( s.contains( QString::fromUtf8( "Vers des numéros spéciaux" ) ) )
			lSpecial2->setText( sum );
		else if ( s.contains( "TOTAL" ) )
			lTotal2->setText( sum );
	}
}
//
void UIInvoice::readTelevision( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( QString( "An error occurs getting file: %1" ).arg( mUrlTelevision ) );
		return;
	}
	
	// update gui
	QTreeWidgetItem* it = new QTreeWidgetItem( twTelevision );
	it->setFlags( Qt::ItemIsEnabled );
	it->setText( 0, "Vos chaines / packs" );
	it->setText( 1, "Tarifs" );
	it->setTextAlignment( 1, Qt::AlignHCenter );
	
	// parse
	QString ba = removeEuros( QString::fromLatin1( buffer ) );
	
	QString key = "<div  id=\"content\" class=\"abonnement\">";
	int pos = ba.indexOf( key );
	ba = ba.mid( pos );
	
	key = "<div class=\"clearer\">";
	pos = ba.indexOf( key );
	ba = ba.left( pos ).trimmed();
	
	QRegExp rx = QRegExp( "<tr>(.*)</tr>" );
	rx.setMinimal( true );
	pos = 0;
	QTreeWidgetItem* mLastItem = 0;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// get tr tag
		const QString s = rx.cap( 0 ).simplified();
		if ( s.contains( "<th>" ) )
			continue;
		if ( QString( s ).remove( " " ) == "<tr></tr>" )
			continue;
		// get values
		QStringList mValues;
		QRegExp rxValues( ">\\s*(([\\w\\s/,.'+-]+))\\s*<" );
		int posV = 0;
		while ( ( posV = rxValues.indexIn( s, posV ) ) != -1 )
		{
			posV += rxValues.matchedLength();
			const QString v = rxValues.cap( 1 ).trimmed();
			if ( !v.isEmpty() )
				mValues << v;
		}
		if ( mValues.isEmpty() )
			continue;
		// create item
		const bool isAboOn = s.contains( "tv_abo_on.gif" );
		QTreeWidgetItem* it = isAboOn ? new QTreeWidgetItem( twTelevision ) : mLastItem;
		// fill item
		if ( isAboOn )
		{
			it->setText( 0, mValues.first() );
			it->setToolTip( 0, mValues.first() );
			it->setTextColor( 1, Qt::red );
			it->setTextAlignment( 1, Qt::AlignRight );
			it->setText( 1, fixMontant( mValues.last() ) );
			it->setToolTip( 1, mValues.first() );
		}
		else if ( it )
		{
			it->setToolTip( 0, mValues.first() );
			it->setToolTip( 1, mValues.first() );
		}
		// remember last item
		mLastItem = it;
	}
	
	if ( twTelevision->topLevelItemCount() == 1 )
	{
		QTreeWidgetItem* it = twTelevision->topLevelItem( 0 );
		it->setFirstColumnSpanned( true );
		it->setTextAlignment( 0, Qt::AlignHCenter );
		it->setText( 0, "Aucun abonnement en cours." );
	}
}
//
void UIInvoice::on_twInvoices_itemDoubleClicked( QTreeWidgetItem* item, int /*column*/ )
{
	QUrl url( REQUESTER->freeBaseUrl().append( QString( "conso/%1" ).arg( item->data( 0, Qt::UserRole ).toString() ) ) );
	if ( !misc::openUrl( url ) )
		UIMAIN->alert( tr( "Can't open: %1" ).arg( url.toString() ) );
}
//
void UIInvoice::on_fgShowDetails_clicked()
{
	QUrl url( REQUESTER->freeBaseUrl().append( QString( "conso/%1" ).arg( fgShowDetails->data().toString() ) ) );
	if ( !misc::openUrl( url ) )
		UIMAIN->alert( tr( "Can't open: %1" ).arg( url.toString() ) );
}

void UIInvoice::saveInvoices()
{
	QList<QTreeWidgetItem*> mItems = twInvoices->selectedItems();
	if ( mItems.count() == 0 )
	{
		UIMAIN->alert( QString::fromUtf8( "Vous devez séléctionnez au moins 1 mois." ) );
		return;
	}
	QString path = Options::current()->value( mInvoiceSavePathSettings ).toString();
	path = QFileDialog::getExistingDirectory( this, "Choisissez un dossier pour la sauvegarde de vos factures", path );
	if ( path.isEmpty() )
		return;
	Options::current()->setValue( mInvoiceSavePathSettings, path );
	foreach ( QTreeWidgetItem* item, mItems )
	{
		const QUrl url( REQUESTER->freeBaseUrl().append( QString( "conso/%1" ).arg( item->data( 0, Qt::UserRole ).toString() ) ) );
		const int mois = url.queryItemValue( "mois" ).toInt();
		mRequests[ REQUESTER->requestUrl( url ) ] = mois;
	}
}

void UIInvoice::customRequestFinished( int id, bool error, QIODevice* device )
{
	// check if id is for downlaoding invoice
	if ( !mRequests.contains( id ) )
		return;
	// get year/month
	const int mois = mRequests.value( id );
	// remove request
	mRequests.remove( id );
	// check if error
	if ( error )
	{
		UIMAIN->alert( QString::fromUtf8( "Une erreur est survenue lors de la recupération de la facture %1" ).arg( mRequests.value( id ) ) );
		return;
	}
	// get/create destination path
	const QString path = Options::current()->value( mInvoiceSavePathSettings ).toString();
	if ( !QDir( path ).exists() )
	{
		if ( !QDir().mkpath( path ) )
		{
			UIMAIN->alert( QString::fromUtf8( "Impossible de créer le dossier de sauvegarde:\n(%1)" ).arg( path ) );
			return;
		}
	}
	// generate filename
	QDate date = QDate::fromString( QString::number( mois ), "yyyyMM" );
	QString fileName = QString( "%1/Facture Free Haut Debit - %2.html" ).arg( path ).arg( date.toString( "MMMM yyyy" ) );
	// create / fill file
	QFile file( fileName );
	if ( !file.open( QIODevice::WriteOnly ) )
	{
		UIMAIN->alert( QString::fromUtf8( "Impossible d'ouvrir le fichier pour écriture:\n(%1)" ).arg( fileName ) );
		return;
	}
	file.resize( 0 );
	file.write( device->readAll() );
	file.close();
}
