/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : mardi 25 juillet 2006 - 19h39
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox@gmail.com )
* FILENAME     : UIMain.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIMain.h"
#include "main.h"
#include "misc.h"
#include "Options.h"

#include "UIPublicity.h"
//
#include "UIAbout.h"
#include "UIAlert.h"
#include "UIMonitor.h"
#include "UIConfigure.h"
#include "UIWifiRooter.h"
#include "UIPhone.h"
#include "UITelevision.h"
#include "UIInvoice.h"
#include "UILineFeature.h"
#include "UISubscribe.h"
#include "UIRate.h"
#include "UIEmail.h"
//
#include "UINetwork.h"
//
#include "Recorder.h"
//
#include <QCloseEvent>
#include <QMenu>
#include <QDate>
#include <QErrorMessage>
//
QPointer<UIMain> UIMain::mSelf = 0L;
//
UIMain* UIMain::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UIMain( p );
	return mSelf;
}
//
UIMain::UIMain( QWidget* p )
	: fgDialog( p ), mInit( false ), mSmall( false )
{
	setupUi( this );
	fixMacWidgets();
	setSmall( mSmall );

	// Inits
	Recorder::self();
	
	// system trayicon is only available since qt 4.2-tp1
#if QT_VERSION >= 0x040200
	// creating system tray
	mTray = new QSystemTrayIcon( QIcon( ":/systray/Pictures/systray/icon.png" ), this );
	// set trayicon tooltip
	mTray->setToolTip( PROGRAM_TITLE );
	// create trayicon menu
	mTrayMenu = new QMenu( this );
	// assign trayicon menu
	mTray->setContextMenu( mTrayMenu );
	mTrayEmails = mTrayMenu->addMenu( QIcon( ":/systray/Pictures/systray/mailbox.png" ), "E-mails" );
	// fill menu
	mTrayMenu->addAction( QIcon( ":/systray/Pictures/systray/invoices.png" ), "Factures", this, SLOT( on_fgInvoice_clicked() ) );
	mTrayMenu->addMenu( mTrayEmails );
	QStringList a = misc::emailAccounts();
	if ( a.count() > 0 )
	{
		foreach ( QString s, a )
			mTrayEmails->addAction( s );
		connect( mTrayEmails, SIGNAL( triggered( QAction* ) ), this, SLOT( openMailBox( QAction* ) ) );
	}
	else
	mTrayEmails->addAction( QString::fromUtf8( "Aucun e-mail configuré"), this, SLOT( on_fgEmail_clicked() ) );
	mTrayMenu->addAction( QIcon( ":/systray/Pictures/systray/rate.png" ), QString::fromUtf8( "Débit"), this, SLOT( on_fgRate_clicked() ) );
	mTrayMenu->addAction( QIcon( ":/systray/Pictures/systray/subscribe.png" ), QString::fromUtf8( "Caractéristiques"), this, SLOT( on_fgSubscribe_clicked() ) );
	mTrayMenu->addAction( QIcon( ":/systray/Pictures/systray/rooter.png" ), QString::fromUtf8( "Routeur / Wifi"), this, SLOT( on_fgRooterWifi_clicked() ) );
	mTrayMenu->addAction( QIcon( ":/systray/Pictures/systray/telephony.png" ), QString::fromUtf8( "Téléphonie"), this, SLOT( on_fgPhone_clicked() ) );
	mTrayMenu->addAction( QIcon( ":/systray/Pictures/systray/networkinformations.png" ), QString::fromUtf8( "Infos réseau"), this, SLOT( on_fgNetwork_clicked() ) );
	mTrayMenu->addAction( QIcon( ":/systray/Pictures/systray/parental.png" ), QString::fromUtf8( "Télévision" ), this, SLOT( on_fgTV_clicked() ) );
	mTrayMenu->addSeparator();
	mTrayMenu->addAction( QIcon( ":/systray/Pictures/systray/exit.png" ), "Quitter", qApp, SLOT( quit() ) );
	// setting trayicon background
	mTrayMenu->setAutoFillBackground( false );
	//mTrayMenu->setContentsMargins( 5, 40, 7, 0 );
	//setWidgetTransparent( mTrayMenu, QPixmap( ":/systray/Pictures/systray/background.png" ), false, mTrayMenu->backgroundRole() );
	// signal
	connect( mTray, SIGNAL( activated( QSystemTrayIcon::ActivationReason ) ), this, SLOT( activated( QSystemTrayIcon::ActivationReason ) ) );
	// show trayicon
	mTray->show();
#endif
	setShowLineFeatures( showLineFeatures() );
}
//
void UIMain::showEvent( QShowEvent* e )
{
	fgDialog::showEvent( e );
	if ( !mInit )
	{
		if ( !misc::identify().isValid() )
			on_fgConfigure_clicked();
		else
			refreshCombos();
	}
	if ( Options::current()->value( "/FreeGo/Options/Monitor/AutoStart", false ).toBool() )
		on_fgMonitor_clicked();
}
//
void UIMain::closeEvent( QCloseEvent* )
{
	Options::current()->sync();
	
#if QT_VERSION >= 0x040200
	// free trayicon
	delete mTrayEmails;
	delete mTrayMenu;
	delete mTray;
#endif
}
//
void UIMain::reject()
{
	qApp->setQuitOnLastWindowClosed( true );
	fgDialog::reject();
}
//
#if QT_VERSION >= 0x040200
void UIMain::activated( QSystemTrayIcon::ActivationReason i )
{
	switch ( i )
	{
		case QSystemTrayIcon::Unknown:
		case QSystemTrayIcon::Context:
		case QSystemTrayIcon::DoubleClick:
		case QSystemTrayIcon::MiddleClick:
			break;
		case QSystemTrayIcon::Trigger:
			if ( !isVisible() )
				show();
			break;
	}

}
#endif
//
void UIMain::alert( const QString& s )
{
	UIAlert::alert( 0, s );
}
//
void UIMain::refreshCombos()
{
	// update free accounts list
	cbFreeAccounts->clear();
	cbFreeAccounts->addItems( misc::freeAccounts() );
	cbFreeAccounts->setCurrentIndex( cbFreeAccounts->findText( misc::identify().Login ) );
	// update email accounts list
	cbEmailAccounts->clear();
	cbEmailAccounts->addItems( misc::emailAccounts() );
	if ( cbEmailAccounts->count() == 0 )
		cbEmailAccounts->addItem( "Aucun compte email." );
}
//
void UIMain::updateProgram( const QString& s, bool b )
{
	if ( b )
	{
		alert( QString( "Can't get %1 update : %2" ).arg( PROGRAM_NAME, s ) );
		return;
	}
	// get absolute file path
	QString f = QFileInfo::QFileInfo( s ).canonicalFilePath();
	// rename old app
	QString m;
	if ( !QFile::rename( QApplication::applicationFilePath(), QApplication::applicationFilePath().append( ".bak.%1" ).arg( QDate::currentDate().toString( "dddd dd MMMM yyyy" ) ) ) )
		m.append( "Impossible de cr�er une sauvegarde de l'application actuelle.<br>" );
	// rename new app
	if ( !QFile::rename( f, QApplication::applicationFilePath() ) )
		m.append( "Impossible d'appliquer la mise � jour.<br>" );
	else
		m.append( "Mise � jour de FreeGo accomplie.<br>Veuillez relancer FreeGo." );
	//
	alert( m );
}
//
void UIMain::openMailBox( QAction* a )
{
	misc::openMailBox( a->text() );
}
//
void UIMain::setSmall( bool b )
{
	mSmall = b;
	if ( mSmall )
	{
		setBackgroundPixmap( QPixmap( ":/principale/Pictures/principale/freegoV4-reduit.png" ) );
		//
		fgSmall->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-mini01.png" ) );
		fgSmall->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-mini02.png" ) );
		//
		fgRooterWifi->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_01.png" ) );
		fgRooterWifi->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_01.png" ) );
		fgRooterWifi->move( 270, 85 );
		//
		fgPhone->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_02.png" ) );
		fgPhone->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_02.png" ) );
		fgPhone->move( 60, 85 );
		//
		fgTV->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_03.png" ) );
		fgTV->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_03.png" ) );
		fgTV->move( 144, 85 );
		//
		fgInvoice->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_04.png" ) );
		fgInvoice->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_04.png" ) );
		fgInvoice->move( 18, 85 );
		//
		fgSubscribe->move( 186, 85 );
		setShowLineFeatures( showLineFeatures() );
		//
		fgRate->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_06.png" ) );
		fgRate->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_06.png" ) );
		fgRate->move( 228, 85 );
		//
		fgNetwork->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_09.png" ) );
		fgNetwork->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_09.png" ) );
		fgNetwork->move( 312, 85 );
		//
		fgEmail->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_08.png" ) );
		fgEmail->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_08.png" ) );
		fgEmail->move( 102, 85 );
	}
	else
	{
		setBackgroundPixmap( QPixmap( ":/principale/Pictures/principale/freegoV4.png" ) );
		//
		fgSmall->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-normal01.png" ) );
		fgSmall->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-normal02.png" ) );
		//
		fgRooterWifi->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-routeurwifi.png" ) );
		fgRooterWifi->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-routeurwifi02.png" ) );
		fgRooterWifi->move( 188, 166 );
		//
		fgPhone->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-telephonie.png" ) );
		fgPhone->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-telephonie02.png" ) );
		fgPhone->move( 26, 127 );
		//
		fgTV->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-television.png" ) );
		fgTV->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-television02.png" ) );
		fgTV->move( 26, 205 );
		//
		fgInvoice->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-factures.png" ) );
		fgInvoice->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-factures02.png" ) );
		fgInvoice->move( 26, 88 );
		//
		setShowLineFeatures( showLineFeatures() );
		fgSubscribe->move( 188, 88 );
		//
		fgRate->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-testdebit.png" ) );
		fgRate->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-testdebit02.png" ) );
		fgRate->move( 188, 127 );
		//
		fgNetwork->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-infos.png" ) );
		fgNetwork->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-infos02.png" ) );
		fgNetwork->move( 188, 205 );
		//
		fgEmail->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-email.png" ) );
		fgEmail->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-email02.png" ) );
		fgEmail->move( 26, 166 );
	}
}
//
bool UIMain::showLineFeatures() const
{
	return !Options::current()->value( "/FreeGo/Options/ShowSubscribe", false ).toBool();
}
//
void UIMain::setShowLineFeatures( bool show )
{
	//fgSubscribe
	if ( mSmall )
	{
		if ( show )
		{
			fgSubscribe->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_05.png" ) );
			fgSubscribe->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_05.png" ) );
		}
		else
		{
			fgSubscribe->setButtonNormalPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn_05.png" ) );
			fgSubscribe->setButtonPushedPixmap( QPixmap( ":/icones/Pictures/principale/icones/btn-over_05.png" ) );
		}
	}
	else
	{
		if ( show )
		{
			fgSubscribe->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-caracteristiques.png" ) );
			fgSubscribe->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-caracteristiques02.png" ) );
		}
		else
		{
			fgSubscribe->setButtonNormalPixmap( QPixmap( ":/principale/Pictures/principale/btn-suivi.png" ) );
			fgSubscribe->setButtonPushedPixmap( QPixmap( ":/principale/Pictures/principale/btn-suivi02.png" ) );
		}
	}
}
//
void UIMain::on_fgLogo_clicked()
{
	UIAbout::self( this )->exec();
}
//
void UIMain::on_fgSmall_clicked()
{
	setSmall( !mSmall );
}
//
void UIMain::on_fgSystray_clicked()
{
#if QT_VERSION >= 0x040200
	if ( isVisible() )
		hide();
#endif
}
//
void UIMain::on_cbFreeAccounts_activated( const QString& s )
{
	misc::setCurrentFreeAccount( s );
}
//
void UIMain::on_fgGoEmail_clicked()
{
	if ( cbEmailAccounts->currentText().contains( '@' ) )
	misc::openMailBox( cbEmailAccounts->currentText() );
}
//
void UIMain::on_fgMonitor_clicked()
{
	UIMonitor::self( this )->show();
}
//
void UIMain::on_fgConfigure_clicked()
{
	UIConfigure::self( this )->exec();
	refreshCombos();
	setShowLineFeatures( showLineFeatures() );
}
//
void UIMain::on_fgRooterWifi_clicked()
{
	UIWifiRooter::self( this )->show();
}
//
void UIMain::on_fgPhone_clicked()
{
	UIPhone::self( this )->show();
}
//
void UIMain::on_fgTV_clicked()
{
	switch ( misc::identify().Connexion )
	{
		case FreeBoxNDI:
		case FreeBoxTotal:
			UITelevision::self( this )->show();
			break;
		case FreeBoxIP:
		case FreeBoxIPTotal:
			alert( QString::fromUtf8( "Votre offre ADSL ne vous permet pas de bénéficier du service multiposte." ) );
			break;
	}
}
//
void UIMain::on_fgInvoice_clicked()
{
	UIInvoice::self( this )->show();
}
//
void UIMain::on_fgSubscribe_clicked()
{
	if ( showLineFeatures() )
		UILineFeature::self( this )->show();
	else
		UISubscribe::self( this )->show();
}

//
void UIMain::on_fgRate_clicked()
{
	UIRate::self( this )->show();
}
//

void UIMain::on_fgNetwork_clicked()
{
	UINetwork::self( this )->show();
}
//
void UIMain::on_fgEmail_clicked()
{
	QWidget* dialog = UIEmail::self( this );
	connect( dialog, SIGNAL( destroyed( QObject* ) ), this, SLOT( refreshCombos()) );
	dialog->show();
}
