/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : dimanche 01 octobre 2006 - 16h01
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UIPublicity.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIPublicity.h"
#include "misc.h"
#include "main.h"

#include <QFileInfo>
#include <QDebug>
#include <QtGui>

QPointer<UIPublicity> UIPublicity::mSelf = 0L;

UIPublicity* UIPublicity::self( const QString& s, const QUrl& u, QWidget* p )
{
	if ( !mSelf )
		mSelf = new UIPublicity( p );
	mSelf->setMode( s );
	mSelf->openUrl( u );
	return mSelf;
}

UIPublicity::UIPublicity( QWidget* p )
	: fgDialog( p )
{
	setupUi( this );
	fixMacWidgets();
	setAttribute( Qt::WA_DeleteOnClose );
	connect( REQUESTER, SIGNAL( requestFinished( int, bool ) ), this, SLOT( requestFinished( int, bool ) ) ); 
}

void UIPublicity::hideEvent( QHideEvent* event )
{
	fgDialog::hideEvent( event );
	emit hidden();
}

void UIPublicity::reject()
{
	foreach ( QTemporaryFile* f, pixmaps.values() )
		delete f;
	fgDialog::reject();
}

void UIPublicity::on_tbSource_anchorClicked( const QUrl& u )
{
	if ( misc::openUrl( u ) )
		tbSource->setSource( tbSource->source() );
	else
		UIMAIN->alert( QString( "Impossible d'ouvrir cette url: %1" ).arg( u.toString() ) );
}

void UIPublicity::setMode( const QString& s )
{
	setBackgroundPixmap( QPixmap( QString( ":/annonce/Pictures/annonce/publigo-%1.png" ).arg( s ) ) );
	tbSource->resize( s.left( s.indexOf( "x" ) ).toInt(), s.mid( s.indexOf( "x" ) +1 ).toInt() );
	fgClose->move( width() -( fgClose->width() +4 ), 4 );
}

void UIPublicity::openUrl( const QUrl& u )
{
	QApplication::setOverrideCursor( Qt::WaitCursor );
	mUrl = u;
	REQUESTER->requestUrl( this, "readPage", u, "GET", pHttpRequest::rtByteArray );
}

void UIPublicity::readPage( const QByteArray& buffer, bool b )
{
	if ( buffer.isEmpty() || b )
	{
		QApplication::restoreOverrideCursor();
		UIMAIN->alert( "Impossible d'ouvrir la source." );
		close();
		return;
	}

	// getting images tag
	mBuffer.setData( buffer );
	QString data = mBuffer.data();
	int startPos = 0, endPos = 0;
	QString image;
	images.clear();
	while ( ( startPos = data.indexOf( "<img", endPos, Qt::CaseInsensitive ) ) != -1 )
	{
		endPos = data.indexOf( ">", startPos );
		images << data.mid( startPos, endPos -startPos +1 ).simplified();
		startPos++;
	}
	
	qWarning() << buffer;

	// downloads images
	int id;
	QString url;
	QTemporaryFile* file;
	foreach ( image, images )
	{
		QHash<QString, QString> imgValues =misc::parseHTMLTag( image );
		image = QFileInfo( mUrl.toString() ).path() +"/" +imgValues.value( "src" );
		QString suffixe = QFileInfo( imgValues.value( "src" ) ).completeSuffix();
		if ( !urlPixmaps.values().contains( imgValues.value( "src" ) ) )
		{
			file = new QTemporaryFile( QString( "%1/FreeGoImagesXXXXXX.%2" ).arg( QDir::tempPath() ).arg( suffixe ) );
			file->open();
			id = REQUESTER->get( image, file );
			urlPixmaps[ id ] = imgValues.value( "src" );
		}
	}
}

void UIPublicity::requestFinished( int id, bool error )
{
	if ( !urlPixmaps.contains( id ) )
		return;
	if ( !error )
	{
		pixmaps[ urlPixmaps[ id ] ] = qobject_cast<QTemporaryFile*>( REQUESTER->currentDestinationDevice() );
		pixmaps[ urlPixmaps[ id ] ]->seek( 0 );
	}
	urlPixmaps.remove( id );
	if ( urlPixmaps.isEmpty() )
		pixmapsDone( false );
}

void UIPublicity::pixmapsDone( bool /*error*/ )
{
	QString data = QString::fromUtf8( mBuffer.data() );
	foreach ( const QString& key, pixmaps.keys() )
		data = data.replace( key, pixmaps.value( key )->fileName() );
	mBuffer.setData( data.toUtf8() );
	tbSource->setHtml( data );
	QApplication::restoreOverrideCursor();
	
	qWarning() << data;
}

