/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : dimanche 06 aot 2006 - 19h15
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com ), Adrien ( adrien at freego dot fr )
* FILENAME     : UIRate.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIRate.h"
#include "src/misc.h"
#include "src/Options.h"
//
#include <QTimer>
#include <QBuffer>
#include <QTcpSocket>
//
const int FILESIZE = 32;
const int DLTIME = 12000;
const int UPTIME = 8000;
const int BARWIDTH = 336;
QPointer<UIRate> UIRate::mSelf = 0L;
//
UIRate* UIRate::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UIRate( p );
	return mSelf;
}
//
UIRate::UIRate( QWidget* p )
	: fgDialog( p ), mErrorGet( false ), mErrorSend( false )
{
	setupUi( this );
	fixMacWidgets();
	//
	mTimer.setInterval( 10 ); // 100ms
	fgProgressBar->hide();
	mSendSocket = 0;
	dDl = 0;
	dUl = 0;
	mPhase = None;
	//
	connect( fgRate, SIGNAL( clicked() ), this, SLOT( getServer() ) );
	connect( &mTimer, SIGNAL( timeout() ), this, SLOT( timeout() ) );
}
//
UIRate::~UIRate()
{
	QApplication::restoreOverrideCursor();
	disconnect( &mHttp, SIGNAL( done( bool ) ), this, SLOT( printResultGet( bool ) ) );
}
//
void UIRate::getServer()
{
	mPhase = GetServerPhase;
	fgRate->setEnabled( false );
	fgProgressBar->resize( 0, fgProgressBar->height() );
	fgProgressBar->show();
	QApplication::setOverrideCursor( Qt::WaitCursor );
	//
	connect( &mHttp, SIGNAL( requestFinished( int, bool ) ), this, SLOT( printServer( int, bool ) ) );
	mHttp.setHost( "debit.free-go.net" );
	requestServerId = mHttp.get( "/up.php", new QBuffer );
}
//
void UIRate::printServer( int id, bool error)
{
	if ( id != requestServerId )
		return;
	mErrorSend = error;
	mServer.setUrl( qobject_cast<QBuffer*>( mHttp.currentDestinationDevice() )->buffer() );
	if ( mServer.toString().isEmpty() )
		mErrorSend = true;
	delete mHttp.currentDestinationDevice();
	disconnect( &mHttp, SIGNAL( requestFinished( int, bool ) ), this, SLOT( printServer( int, bool ) ) );
	getData();
}
//
void UIRate::getData()
{
	// TODO: disable mail-check-notification during the test, until printResultSend().
	connect( &mHttp, SIGNAL( done( bool ) ), this, SLOT( printResultGet( bool ) ) );
	connect( &mHttp, SIGNAL( dataReadProgress( int, int ) ), this , SLOT( updateProgressBar( int, int ) ) );
	//
	mHttp.get( QString( "/%1m.rnd" ).arg( FILESIZE ), &mBuffer );
	mReceive = 0;
	QTimer::singleShot( DLTIME, &mHttp, SLOT( abort() ) );
	mTimer.start();
	mGetTime.start();
	mPhase = GetPhase;
}
//
void UIRate::printResultGet( bool error ) // end of first phase
{
	Q_UNUSED( error ); // it's usually true due to abort(), so don't care about the error code
	mErrorGet = mReceive == 0; // no bytes received does mean an error though :)
	if ( mGetTime.elapsed() )
		dDl = mReceive / mGetTime.elapsed();
	sendData();
}
//
void UIRate::sendData()
{
	mPhase = SendPhase;
	mSendTime.start();
	if ( !mErrorSend && !mServer.toString().simplified().isEmpty() )
	{
		mSendSocket = new QTcpSocket( this );
		mSendSocket->connectToHost( mServer.host(), mServer.port() );
		if ( !mSendSocket->waitForConnected( 5000 ) )
		{
			mErrorSend = true;
			delete mSendSocket;
			mSendSocket = 0;
		}
		else
		{
			connect( mSendSocket, SIGNAL( disconnected() ), this, SLOT( printResultSend() ) );
			connect( mSendSocket, SIGNAL( bytesWritten( qint64 ) ), this, SLOT( bytesWritten( qint64 ) ) );
			mSend = 0;
			mSendSocket->write( mBuffer.data() );
			QTimer::singleShot( UPTIME, this, SLOT( abortSend() ) );
		}
	}
	if ( mErrorSend )
	{
		mTimer.stop();
		fgProgressBar->resize( BARWIDTH, fgProgressBar->height() );
		printResultSend();
	}
}
//
void UIRate::printResultSend() // end of second phase
{
	if ( mSendTime.elapsed() )
		dUl = mSend / mSendTime.elapsed();
	mTimer.stop();
	fgProgressBar->resize( BARWIDTH, fgProgressBar->height() ); // cheating :)
	//
	QString m;
	if ( !mErrorGet )
		m += QString( "Test descendant : %1 Kbits/s, soit %2 Ko/s." ).arg( (int)(dDl * 8) ).arg( (int)dDl );
	else
		m += QString::fromUtf8("\nErreur lors du test de téléchargement");
	if ( !mErrorSend )
		m += QString( "\nTest remontant : %3 Kbits/s, soit %4 Ko/s." ).arg( (int)(dUl * 8) ).arg( (int)dUl );
	else
		m += QString("\nErreur lors du test remontant");
	lResult->setText( m );
	//
	if ( !mErrorGet )
	{
		QString c;
		if ( dDl < 10 )
			c = QString::fromUtf8("Débit comparable à l'offre Free 56 k." );
		else if ( dDl < 16 )
			c = QString::fromUtf8("Débit comparable à une offre  128 k." );
		else if ( dDl < 70 )
			c = QString::fromUtf8("Débit comparable à l'offre Free ADSL 512." );
		else if ( dDl < 130 )
			c = QString::fromUtf8("Débit comparable à l'offre Free ADSL 1024." );
		else if ( dDl < 260 )
			c = QString::fromUtf8("Débit comparable à l'offre Free ADSL 2 Mbits." );
		else if ( dDl < 580 )
			c = QString::fromUtf8("Débit comparable à l'offre Free ADSL dégroupé ou IP/MAX" );
		else if ( dDl < 980 )
			c = QString::fromUtf8("Débit exceptionnel comparable à une offre de 10 Mbits" );
		else if ( dDl < 18000 )
			c = QString::fromUtf8("Débit exceptionnel correspondant à l'offre ADSL2+." );
		else
			c = QString::fromUtf8("Débit correspondant à l'offre FTTH de Free" );
		lComment->setText( c );
	}
	//
	fgRate->setEnabled( true );
	QApplication::restoreOverrideCursor();
}
//
void UIRate::updateProgressBar( int done, int /*total*/ )
{
	mReceive = done;
}
//
void UIRate::bytesWritten( qint64 bytes )
{
	mSend += bytes;
}
//
void UIRate::abortSend()
{
	mSendSocket->abort();
}
//
void UIRate::timeout()
{
	// [<---get---><--send-->]
	double progress = 0;
	switch ( mPhase )
	{
	case GetPhase:
		progress = (double)mGetTime.elapsed() / 1000.0;
		break;
	case SendPhase:
		progress = ( DLTIME + (double)mSendTime.elapsed() ) / 1000.0;
		break;
	default:
		Q_ASSERT( 0 );
		break;
	}
	double total = ( DLTIME + UPTIME ) / 1000;
	int labelWidth = qRound( progress * BARWIDTH / total );
	fgProgressBar->resize( labelWidth, fgProgressBar->height() );
}
