/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : jeudi 27 juillet 2006 - 18h57
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UISubscribe.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UISubscribe.h"
#include "main.h"
#include "UILineFeature.h"
#include "misc.h"

QPointer<UISubscribe> UISubscribe::mSelf = 0L;

UISubscribe* UISubscribe::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UISubscribe( p );
	return mSelf;
}

UISubscribe::UISubscribe( QWidget* p )
	: fgDialog( p )
{
	setupUi( this );
	fixMacWidgets();
	REQUESTER->login( this, "readLogin" );
}

void UISubscribe::readLogin( QSettings*, bool b )
{
	if ( b )
	{
		UIMAIN->alert( "Impossible de vous identifiez sur free, annulation" );
		deleteLater();
		return;
	}
	REQUESTER->requestUrl( this, "readSubscribe", REQUESTER->freeBaseUrl().append( "suivi/suivi.pl" ), "GET", pHttpRequest::rtByteArray );
}

void UISubscribe::addLine( const QString& text, const QString& pixmap, Qt::GlobalColor color )
{
	QList<QLabel*> labels;
	QLabel* label;
	
	// pixmap
	if ( !pixmap.isEmpty() )
	{
		label = new QLabel( this );
		label->setPixmap( QPixmap( pixmap ) );
		label->setAlignment( Qt::AlignTop | Qt::AlignLeft );
		label->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred ) );
		labels << label;
	}
	
	// text
	label = new QLabel( text, this );
	label->setTextFormat( Qt::RichText );
	label->setAlignment( Qt::AlignTop | Qt::AlignHCenter );
	if ( !pixmap.isEmpty() )
		label->setAlignment( Qt::AlignTop | Qt::AlignLeft );
	QPalette palette( label->palette() );
	palette.setColor( label->foregroundRole(), color );
	QFont font( label->font() );
	if ( color == Qt::red )
		font.setBold( true );
	label->setFont( font );
	label->setPalette( palette );
	connect( label, SIGNAL( linkActivated( const QString& ) ), this, SLOT( linkActivated( const QString& ) ) );
	labels << label;
	
	if ( labels.count() == 1 )
		vlLayout->addWidget( labels.at( 0 ) );
	else
	{
		QHBoxLayout* hbLayout = new QHBoxLayout;
		hbLayout->setSpacing( 16 );
		foreach ( QLabel* l, labels )
			hbLayout->addWidget( l );
		vlLayout->addLayout( hbLayout );
	}
}

void UISubscribe::readSubscribe( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( "Impossible de telecharger le fichier suivi.pl" );
		return;
	}
	
	// parse
	QString ba = QString::fromLatin1( buffer );
	
	QString key = "<div  id=\"content\" class=\"abonnement\">";
	int pos = ba.indexOf( key );
	ba = ba.mid( pos );
	
	key = "<div class=\"clearer\">";
	pos = ba.indexOf( key );
	ba = ba.left( pos ).trimmed();
	
	QRegExp rx( "(<li.*</li>)" );
	rx.setMinimal( true );
	pos = 0;
	bool redFound = false;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// get li tag
		QString s = rx.cap( 0 ).simplified();
		// get etape / phrase
		QRegExp rxValues( "<li(?: class=\"(\\w+)\")?>(?:<span class=\"bold\">Etape (\\d)</span> - )?((.*))</li>" );
		QStringList values;
		QString type;
		if ( rxValues.indexIn( s ) != -1 )
		{
			values = rxValues.capturedTexts();
			values.removeFirst();
			if ( values.at( 1 ).toInt() > 0 )
			{
				type = "bleu";
				if ( redFound )
					type = "gris";
				else if ( values.at( 0 ) == "selected" )
				{
					type = "red";
					redFound = true;
				}
				QString fn = QString( ":/suivi/Pictures/suivi/%1%2.png" ).arg( values.at( 1 ).toInt() ).arg( type );
				addLine( values.at( 2 ), fn, type == "red" ? Qt::red : Qt::black );
			}
			else
				addLine( values.at( 2 ) );
		}		
	}
	
	fixMacWidgets();
}

void UISubscribe::linkActivated( const QString& url )
{
	if ( url.contains( "suivi_techgrrr.pl" ) )
		UILineFeature::self()->show();
	else
	{
		QUrl u( url );
		if ( u.isRelative() )
			u.setPath( REQUESTER->freeBaseUrl().append( "suivi/" ) );
		misc::openUrl( u );
	}
}
